==> sc2env-0.1.14-py2.py3-none-any.whl: sc2env/environments/four_towers_sequential.py <==
import os
import time
import sys
import numpy as np
import csv
import json
import imutil

import numpy as np
import pysc2
from pysc2.agents import base_agent
from pysc2.env import sc2_env
from pysc2.lib import actions, features, units
from pysc2 import maps
from sc2env.representation import *

from sc2env.pysc2_util import register_map


MAP_NAME = 'FourTowerSequentialDecomposed'


class FourTowersSequentialEnvironment():
    def __init__(self):
        maps_dir = os.path.join(os.path.dirname(__file__), '..', 'maps')
        register_map(maps_dir, MAP_NAME)
        self.sc2_env = sc2_env.SC2Env(
          map_name=MAP_NAME,
          players=[sc2_env.Agent(sc2_env.Race.terran)],
          agent_interface_format=features.AgentInterfaceFormat(
              feature_dimensions=features.Dimensions(screen=40, minimap=30),
              use_feature_units=True),
          step_mul=16,
          game_steps_per_episode=0,
          score_index=0,
          visualize=False)
        self.current_obs = None
        self.actions_taken = 0
        self.last_mineral_count = 0
        self.reward = 0
        self.zergling_count = 0
        self.roach_count = 0
        self.last_reward = 0
        self.last2_reward = 0
        self.rewards = []
        self.decomposed_rewards = [0,0]
        self.last_timestep = None

    def action_space():
        from gym.spaces.discrete import Discrete
        return Discrete(2)

    def observation_space():
        return (84, 84, 1)

    def print_current_alerts(self):
        return self.current_obs[3]

    def store_last_reward(self, reward):
        self.last_reward = reward

    def reset(self):
        # Move the camera in any direction
        # This runs the ResetEpisode trigger built into the map
        self.decomposed_rewards = []
        self.rewards = []
        self.last_timestep = self.sc2_env.reset()
        action = actions.FUNCTIONS.move_camera([0, 0])
        self.last_timestep = self.sc2_env.step([action])[0]
        observation, state, reward, done, info = self.unpack_timestep(self.last_timestep)
        self.current_obs = observation
        #state = self.int_map_to_onehot(state)
        state = np.array(state)
        self.actions_taken = 0
        state = observation[3]['feature_screen']
        state = expand_pysc2_to_neural_input(state)
        state = np.reshape(state, (1, -1))
        return state

    def noop(self):
        return actions.FUNCTIONS.no_op()

    def step(self, action):
        done = False
        dead = False

        ### ACTION TAKING ###

        if self.actions_taken == 0 and self.check_action(self.current_obs, 12):
            if action == 0:
                action = actions.FUNCTIONS.Attack_screen("now", [0,0])
            elif action == 1:
                action = actions.FUNCTIONS.Attack_screen("now", [39,0])
            elif action == 2:
                action = actions.FUNCTIONS.Attack_screen("now", [0,39])
            elif action == 3:
                action = actions.FUNCTIONS.Attack_screen("now", [39,39])
            elif action == 4:
                action = actions.FUNCTIONS.no_op()
            else:
                print("Invalid action: check final layer of network")
                action = actions.FUNCTIONS.no_op()
        else:
            action = actions.FUNCTIONS.no_op()

        ####################

        ### STATE PREPARATION ###

        self.last_timestep = self.sc2_env.step([action])[0]

        observation, state, reward, done_null, info = self.unpack_timestep(self.last_timestep)
        self.current_obs = observation

        #state = self.int_map_to_onehot(state)
        state = observation[3]['feature_screen']
        # print(np.unique(np.array(state[6])))
        state = expand_pysc2_to_neural_input(state)
        # print('STATE SHAPE')
        # print(state.shape)
        state = np.reshape(state, (1, -1))
        state = np.array(state)
        # print('STATE SHAPE')
        # print(state.shape)

        #########################

        ### REWARD PREPARATION AND TERMINATION ###

        from s2clientprotocol import sc2api_pb2 as sc_pb
        data = self.sc2_env._controllers[0]._client.send(observation=sc_pb.RequestObservation())
        data = data.observation.raw_data.units

        damageByRoach = 0
        damageByZergling = 0
        damageToRoach = 0
        damageToZergling = 0

        for x in data:
            if x.unit_type == 1922:
                roach_reward = x.health
            if x.unit_type == 1923:
                zergling_reward = x.health
            if x.unit_type == 1924:
                damageByRoach = x.health
            if x.unit_type == 1925:
                damageByZergling = x.health
            if x.unit_type == 1926:
                damageToRoach = x.health
            if x.unit_type == 1927:
                damageToZergling = x.health

        # print("Damage by roach: {}".format(damageByRoach))
        # print("Damage by zergling: {}".format(damageByZergling))
        # print("Damage to roach: {}".format(damageToRoach))
        # print("Damage to zergling: {}".format(damageToZergling))

        total_reward = roach_reward + zergling_reward - 4
        reward = total_reward
        self.reward = total_reward
        self.rewards.append(reward)

        if self.last_reward != self.reward:
            done = True
            if self.last_reward > self.reward:
                dead = True
            else:
                dead = False

        self.last_reward = self.reward

        self.decomposed_rewards.append([roach_reward - 2, zergling_reward - 2, damageByRoach - 2, damageByZergling - 2, damageToRoach - 2, damageToZergling - 2])

        ###########################################

        return state, reward, done, dead, info

    def render(self):
        if self.vis is None:
            return

        obs_image = imutil.show(self.last_timestep.observation['rgb_screen'], filename="test.jpg")
        opts = dict(title = "state", width = 360, height = 350)

        if self.image_window is None:
            self.image_window = self.vis.image(obs_image, opts = opts)
        else:
            self.vis.image(obs_image, opts = opts, win = self.image_window)

    def get_mineral_count(self, obs):
        return 0

    def get_vespene_gas_count(self, obs):
        return 0

    def unpack_timestep(self, timestep):
        observation = timestep
        state = timestep.observation.feature_screen[6]
        reward = timestep.observation.score_cumulative[0]
        done = timestep.last()
        info = {}
        return observation, state, reward, done, info

    def get_available_actions(self, obs):
        return obs.observation.available_actions

    def check_action(self, obs, action):
        return action in self.get_available_actions(obs)

    def get_health_of_unit_type(self, unit_id):
        total_health = 0
        feature_units = self.current_obs[3]['feature_units']
        for unit in feature_units:
            if unit[0] == unit_id:
                total_health += unit[2]
        return total_health

    def get_unit_count_of_type(self, unit_id):
        count = 0
        feature_units = self.current_obs[3]['feature_units']
        for unit in feature_units:
            if unit[0] == unit_id:
                count += 1
        return count

    def int_map_to_onehot(self, x, vocabulary=None):
        location_zergling = np.argwhere(x == 105.)
        location_roach = np.argwhere(x == 110.)

        if location_zergling.shape[0] == 0 or location_roach.shape[0] == 0:
            onehot = [0,0,0,0,0,0,0,0]
        else:
            onehot = [location_zergling[0][0], location_zergling[0][1], location_roach[0][0], location_roach[0][1], location_roach[1][0], location_roach[1][1], location_roach[2][0], location_roach[2][1]]

        onehot = np.reshape(onehot, [1, 8])
        return onehot

==> sc2env-0.1.16-py2.py3-none-any.whl: sc2env/environments/four_towers_sequential.py <==
import os
import time
import sys
import numpy as np
import csv
import json
import imutil

import numpy as np
import pysc2
from pysc2.agents import base_agent
from pysc2.env import sc2_env
from pysc2.lib import actions, features, units
from pysc2 import maps
#from sc2env.representation import *
from sc2env.utility import *

from sc2env.pysc2_util import register_map


MAP_NAME = 'FourTowerSequentialDecomposed'


class FourTowersSequentialEnvironment():
    def __init__(self):
        maps_dir = os.path.join(os.path.dirname(__file__), '..', 'maps')
        register_map(maps_dir, MAP_NAME)
        self.sc2_env = sc2_env.SC2Env(
          map_name=MAP_NAME,
          players=[sc2_env.Agent(sc2_env.Race.terran)],
          agent_interface_format=features.AgentInterfaceFormat(
              feature_dimensions=features.Dimensions(screen=40, minimap=30),
              use_feature_units=True),
          step_mul=16,
          game_steps_per_episode=0,
          score_index=0,
          visualize=False)
        self.current_obs = None
        self.actions_taken = 0
        self.last_mineral_count = 0
        self.reward = 0
        self.zergling_count = 0
        self.roach_count = 0
        self.last_reward = 0
        self.last2_reward = 0
        self.rewards = []
        self.decomposed_rewards = [0,0]
        self.last_timestep = None

    def action_space():
        from gym.spaces.discrete import Discrete
        return Discrete(2)

    def observation_space():
        return (84, 84, 1)

    def print_current_alerts(self):
        return self.current_obs[3]

    def store_last_reward(self, reward):
        self.last_reward = reward

    def reset(self):
        # Move the camera in any direction
        # This runs the ResetEpisode trigger built into the map
        self.decomposed_rewards = []
        self.rewards = []
        self.last_timestep = self.sc2_env.reset()
        action = actions.FUNCTIONS.move_camera([0, 0])
        self.last_timestep = self.sc2_env.step([action])[0]
        observation, state, reward, done, info = self.unpack_timestep(self.last_timestep)
        self.current_obs = observation
        #state = self.int_map_to_onehot(state)
        state = np.array(state)
        self.actions_taken = 0
        state = observation[3]['feature_screen']
        # state = expand_pysc2_to_neural_input(state)
        input_screen_feature = {'PLAYER_RELATIVE': [1,3,16], 'UNIT_TYPE':SIMPLE_SC2_UNITS, 'HIT_POINT':0, 'HIT_POINT_RATIO': 0, 'UNIT_DENSITY':0}
        state = getOneHotState(state, input_screen_feature)

        state = np.reshape(state, (1, -1))
        return state

    def noop(self):
        return actions.FUNCTIONS.no_op()

    def step(self, action):
        done = False
        dead = False

        ### ACTION TAKING ###

        if self.actions_taken == 0 and self.check_action(self.current_obs, 12):
            if action == 0:
                action = actions.FUNCTIONS.Attack_screen("now", [0,0])
            elif action == 1:
                action = actions.FUNCTIONS.Attack_screen("now", [39,0])
            elif action == 2:
                action = actions.FUNCTIONS.Attack_screen("now", [0,39])
            elif action == 3:
                action = actions.FUNCTIONS.Attack_screen("now", [39,39])
            elif action == 4:
                action = actions.FUNCTIONS.no_op()
            else:
                print("Invalid action: check final layer of network")
                action = actions.FUNCTIONS.no_op()
        else:
            action = actions.FUNCTIONS.no_op()

        ####################

        ### STATE PREPARATION ###

        self.last_timestep = self.sc2_env.step([action])[0]

        observation, state, reward, done_null, info = self.unpack_timestep(self.last_timestep)
        self.current_obs = observation

        #state = self.int_map_to_onehot(state)
        state = observation[3]['feature_screen']
        # print(np.unique(np.array(state[6])))
        #state = expand_pysc2_to_neural_input(state)
        input_screen_feature = {'PLAYER_RELATIVE': [1,3,16], 'UNIT_TYPE':SIMPLE_SC2_UNITS, 'HIT_POINT':0, 'HIT_POINT_RATIO': 0, 'UNIT_DENSITY':0}
        state = getOneHotState(state, input_screen_feature)
        # print('STATE SHAPE')
        # print(state.shape)
        #state shape (20, 40, 40)
        state = np.reshape(state, (1, -1))
        state = np.array(state)
        # print('STATE SHAPE')
        # print(state.shape)
        #state shape (1, 32000)

        #########################

        ### REWARD PREPARATION AND TERMINATION ###

        from s2clientprotocol import sc2api_pb2 as sc_pb
        data = self.sc2_env._controllers[0]._client.send(observation=sc_pb.RequestObservation())
        data = data.observation.raw_data.units

        damageByRoach = 0
        damageByZergling = 0
        damageToRoach = 0
        damageToZergling = 0

        for x in data:
            if x.unit_type == 1922:
                roach_reward = x.health
            if x.unit_type == 1923:
                zergling_reward = x.health
            if x.unit_type == 1924:
                damageByRoach = x.health
            if x.unit_type == 1925:
                damageByZergling = x.health
            if x.unit_type == 1926:
                damageToRoach = x.health
            if x.unit_type == 1927:
                damageToZergling = x.health

        # print("Damage by roach: {}".format(damageByRoach))
        # print("Damage by zergling: {}".format(damageByZergling))
        # print("Damage to roach: {}".format(damageToRoach))
        # print("Damage to zergling: {}".format(damageToZergling))

        total_reward = roach_reward + zergling_reward - 4
        reward = total_reward
        self.reward = total_reward
        self.rewards.append(reward)

        if self.last_reward != self.reward:
            done = True
            if self.last_reward > self.reward:
                dead = True
            else:
                dead = False

        self.last_reward = self.reward

        self.decomposed_rewards.append([roach_reward - 2, zergling_reward - 2, damageByRoach - 2, damageByZergling - 2, damageToRoach - 2, damageToZergling - 2])

        ###########################################

        return state, reward, done, dead, info

    def render(self):
        if self.vis is None:
            return

        obs_image = imutil.show(self.last_timestep.observation['rgb_screen'], filename="test.jpg")
        opts = dict(title = "state", width = 360, height = 350)

        if self.image_window is None:
            self.image_window = self.vis.image(obs_image, opts = opts)
        else:
            self.vis.image(obs_image, opts = opts, win = self.image_window)

    def get_mineral_count(self, obs):
        return 0

    def get_vespene_gas_count(self, obs):
        return 0

    def unpack_timestep(self, timestep):
        observation = timestep
        state = timestep.observation.feature_screen[6]
        reward = timestep.observation.score_cumulative[0]
        done = timestep.last()
        info = {}
        return observation, state, reward, done, info

    def get_available_actions(self, obs):
        return obs.observation.available_actions

    def check_action(self, obs, action):
        return action in self.get_available_actions(obs)

    def get_health_of_unit_type(self, unit_id):
        total_health = 0
        feature_units = self.current_obs[3]['feature_units']
        for unit in feature_units:
            if unit[0] == unit_id:
                total_health += unit[2]
        return total_health

    def get_unit_count_of_type(self, unit_id):
        count = 0
        feature_units = self.current_obs[3]['feature_units']
        for unit in feature_units:
            if unit[0] == unit_id:
                count += 1
        return count

    def int_map_to_onehot(self, x, vocabulary=None):
        location_zergling = np.argwhere(x == 105.)
        location_roach = np.argwhere(x == 110.)

        if location_zergling.shape[0] == 0 or location_roach.shape[0] == 0:
            onehot = [0,0,0,0,0,0,0,0]
        else:
            onehot = [location_zergling[0][0], location_zergling[0][1], location_roach[0][0], location_roach[0][1], location_roach[1][0], location_roach[1][1], location_roach[2][0], location_roach[2][1]]

        onehot = np.reshape(onehot, [1, 8])
        return onehot
